/*
 * Created on Sep 11, 2003
 */
package isac.bridge;

import isac.util.Formula;
import isac.util.ICalcIterator;
import isac.util.IToCalc;
import isac.util.IToUser;
import isac.util.ModSpec;
import isac.util.Tactic;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.Serializable;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Vector;

/**
 * A CalcTree represents a Calculation: It is a tree which consists
 * of one Model and abitrarily many Formulas and Tactics.
 * SubCalculations are also part of this 
 * This is the main interface between the Bridge and the Dialog
 * @author rgradisc
 */
public class CalcTree implements IToCalc, Serializable {

  private int id;
  private Vector listeners;

  private Socket socket = null;
  private PrintWriter out = null;
  private BufferedReader in = null;

  /**
  Start a new calculation. In case of failure, 
  the constructor may change the formalization object to hint at incomplete or
  contradictory specifications. 
  <br> Under normal circumstances, only complete Modspecs should be passed. 
  <br> Check every ModSpec for completeness (with the
  static method checkModSpec() before constructing a CalcTree
  * @param modSpec The ModSpec for this CalcTree
  */
  public CalcTree(ModSpec modSpec, String host, int port) {
    // create socket connection
    listeners = new Vector();
    try {
      socket = new Socket(host, port);
      out = new PrintWriter(socket.getOutputStream(), true);
      in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
    } catch (UnknownHostException e) {
      System.out.println("Unknown host\n");
    } catch (IOException e) {
      System.out.println("No I/O\n");
    }
    //<FIXXME>
    sendToBridge("states:= [];addUser 1;");
    //</FIXXME>
    /*String s =
      sendToBridge(
        "CalcTree "
          + "[([\"equality (1+-1*2+x=0)\", \"solveFor x\",\"solutions L\"],"
          + "(\"Test.thy\","
          + "[\"linear\",\"univariate\",\"equation\",\"test\"],"
          + "[\"Test\",\"solve_linear\"]))];");
    */
    StringBuffer ctorSB = new StringBuffer("CalcTree [([\"");
    ctorSB.append(modSpec.getGiven().getFormulas()[0].getText());
    ctorSB.append("\",\"");
    ctorSB.append(modSpec.getWhere().getFormulas()[0].getText());
    ctorSB.append("\",\"");
    ctorSB.append(modSpec.getFind().getFormulas()[0].getText());
    ctorSB.append("\"],(\"");
    ctorSB.append(modSpec.getTheory().getText()[0]);
    ctorSB.append("\",[");
    String[] sArr = modSpec.getProblem().getText();
    for (int i = 0; i < sArr.length; i++) {
      ctorSB.append("\"" + sArr[i] + "\",");
    }
    ctorSB.deleteCharAt(ctorSB.length() - 1);
    ctorSB.append("],[");

    sArr = modSpec.getMethod().getText();
    for (int i = 0; i < sArr.length; i++) {
      ctorSB.append("\"" + sArr[i] + "\",");
    }
    ctorSB.deleteCharAt(ctorSB.length() - 1);

    ctorSB.append("]))];");
    String s = sendToBridge(ctorSB.toString());
    //<FIXXME>
    this.id = 1;
    //</FIXXME>
  }

  /**
  This is a static method to check a ModSpec object for completeness 
  and consistency. 
  <p>This Method may change the passed ModSpec object to hint at 
  incomplete or contradictory specifications.
  * @param modSpec The ModSpec to be changed
  * @return success value: 0 == success, else no success
  */
  public static int checkModSpec(ModSpec modSpec) {
    return 0;
  }

  /**
  Attach another dialog to the existing calculation tree. 
  * @param dlg The new Dialog to be attatched to this CalcTree
  */
  public void attach(/*Dialog dlg -- tbd*/
  ) {
  }

  /**
  Get a new iterator referencing the root element of the calculation tree. 
  <p> The functionality for navigating the calculation 
  is implemented in the CalcIterator object.
  * @return new instance of CalcIterator
  */
  public ICalcIterator iterator() {
    return new CalcIterator(this);
  }

  /**
  Makes the passed formula the active formula, i.e. the formula 
  the next tactic is applied to.
  * @param newActiveFormula
  */
  public void moveActiveFormula(CalcIterator newActiveFormula) {
    //String s = sendToBridge("moveActiveFormula 1 1;");
  }

  /**
  Tries to replaces the active formula in the calculation tree by 
  the passed formula. This could fail if no valid derivation from 
  the preceding formula to the (new) active formula can be found. 
  <p>Parts of the calculation after the replaced 
  formula are likely to become invalid.
  * @param newFormula
  * @return The method will return 0 on success or a nonzero value 
  indicating the status otherwise. 
  */
  public int replaceFormula(Formula newFormula) {
    //String s = sendToBridge("replaceFormula 1 1;");
    return 0;
  }

  /**
  Tries to append the passed formula after the active formula. 
  This could fail if no valid derivation from the preceding formula 
  to the (new) active formula can be found. The method will fail as 
  well if the active formula is not the last formula in the (sub)calculation.
  Parts of the calculation might become invalid.
  * @param newFormula
  * @return The method will return 0 on success or a nonzero value 
  indicating the status otherwise.
  */
  public int appendFormula(Formula newFormula) {
    return 0;
  }

  /**
  Set the tactic to be used in the next step of calculation.   
  * @param tactic
  * @return The return value indicates success, failure or probable
  success at a later time.
  */
  public int setNextTactic(Tactic tactic) {
    String s =
      sendToBridge("setNextTactic " + +id + " 1 (" + tactic.getText() + ");");
    return 0;
  }

  /**
  Get the tactic proposed by the mathematics engine for 
  the next step in the calculation.
  * @return proposed tactic
  */
  public Tactic fetchProposedTactic() {
    String s = sendToBridge("fetchProposedTactic " + id + " 1 ;");
    return XMLParser.parseTacticDOM(s);
  }

  /**
  Get a list of tactics applicable to the active formula. 
  * @param scope The filter parameter selects the scope of
  search for applicable tactics: all tactics / all tactics from the 
  current theory / all tactics from the (calculation) method being applied.
  * @return Array with applyable tactics
  */
  public Tactic[] fetchAppliableTactics(int scope) {
    String s = sendToBridge("fetchAppliableTactics " + id + ";");
    return new Tactic[10];
  }

  /**
  Calculate the next n steps starting from the active formula.   
  * @param scope The scope parameter can have following values:
  <ul>
  <li> 1 .. Stay in the current subproblem
  <li> 2 .. Stay in the current subcalculation (e.g. repetitions) 
  <li> 3 .. Stay in the current calculation as a whole. 
  </ul>
  * @param nSteps Specifies the count of steps to be autocalculated. 
  Passing 0 for nSteps will calculate until reaching a final result. 
  * @return The method returns an unique id to identify the request when 
  notifying about updates in the tree.
  */
  public int autoCalculate(int scope, int nSteps) {
    sendToBridge("autoCalculate " + id + ";");
    return 0;
  }

  String sendToBridge(String request) {
    String result = "";
    System.out.println("sent to bridge: " + request);
    out.println(request);
    try {
      result = in.readLine();
    } catch (IOException e) {
      e.printStackTrace();
    }
    System.out.println("received from bridge: " + result);
    return result;
  }
  /**
   * @return
   */
  public int getId() {
    return id;
  }

  public boolean addListener(IToUser listener) {
    if (listeners.contains(listener)) {
      return false;
    } else {
      listeners.add(listener);
    }
    return true;
  }

  /* (non-Javadoc)
   * @see isac.util.IToCalc#getActiveFormula()
   */
  public ICalcIterator getActiveFormula() {
    // TODO Auto-generated method stub
    return null;
  }

}
