/*
 * Created on Aug 13, 2003
 */
package isac.bridge;

/**
 * @author rgradisc
 */
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.net.UnknownHostException;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
//import org.w3c.dom.Document;

class Client extends JFrame implements ActionListener {

  JButton sendButton, connButton, badButton, fileButton;
  JPanel panel, panel2;
  JTextField textField;
  JTextArea textArea;
  JScrollPane scrollPane;

  private Socket socket = null;
  private PrintWriter out = null;

  private ClientInputWorker clientInputWorker;

  //---------------------------------------------------------------
  /**
   * Constructor: starts a new Client
   */
  public Client() {
    // set up the GUI
    textField =
      new JTextField("applytac ###uid### 1 ([1,2,3], Res) (Rewrite (\"addcommute\", \"?a + ?b = ?b + ?a\"));");
    connButton = new JButton("Connect to Bridge");
    sendButton = new JButton("Send to Bridge");
    badButton = new JButton("Send bad msg");
    fileButton = new JButton("Send Batch");
    sendButton.addActionListener(this);
    connButton.addActionListener(this);
    badButton.addActionListener(this);
    fileButton.addActionListener(this);
    textArea = new JTextArea();
    textArea.setLineWrap(true);
    panel = new JPanel();
    panel.setLayout(new BorderLayout());
    panel.setBackground(Color.white);
    scrollPane =
      new JScrollPane(
        textArea,
        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
        JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
    panel2 = new JPanel();
    panel2.add("North", connButton);
    panel2.add("West", sendButton);
    panel2.add("Center", badButton);
    panel2.add("East", fileButton);
    panel.add("North", panel2);
    panel.add("Center", scrollPane);
    panel.add("South", textField);
    getContentPane().add(panel);
  }

  //---------------------------------------------------------------
  /**
   * Called when the user clicks a button
   * @param event The ActionEvent to handle
   */
  public void actionPerformed(ActionEvent event) {
    Object source = event.getSource();

    if (source == connButton) {
      // connect to brigde via socket
      startSocketListenerThread();
    } else if (source == sendButton) {
      sendButtonClick();
    } else if (source == badButton) {
      badButtonClick();
    } else if (source == fileButton) {
      fileButtonClick();
    }
  }
  private void badButtonClick() {
    // Send sml to a endless loop
    if (out != null) {
      out.println(
        "applytac ###uid### 22 ([1,2,3], Res) (Rewrite (\"addcommute\", \"?a + ?b = ?b + ?a\"));");
    } else {
      textArea.append("Connect to Bridge first!\n");
    }
  }
  
  private void sendButtonClick() {
    // Send data over socket
    String text = textField.getText();
    if (out != null) {
      out.println(text);
    } else {
      textArea.append("Connect to Bridge first!\n");
    }
  }
  
  private void fileButtonClick() {
    BufferedReader br;
    try {
      br =
        new BufferedReader(
          new FileReader(new File("/radish/hda-local/isac/smlbatch.txt")));
      while (br.ready()) {
        String line = br.readLine();
        textArea.append(line+'\n');
        out.println(line);
      }
    } catch (FileNotFoundException e) {
      textArea.append("File no found\n");
    } catch (IOException e) {
      textArea.append("IO-Exception\n");
    }
  }

  //---------------------------------------------------------------
  /**
   * Process the response sent from the kernel, usually after a request
   * from the client
   * @param response the response to be processed
   */
  public void processResponse(String response) {
    if (response == null) {
      textArea.append("No I/O: Server seems to have closed socket\n");
    } else {
      System.out.println("Text received :" + response);
      textArea.append(response + "\n");
    }
  }

  //---------------------------------------------------------------
  /**
   * Start a thread (ClientInputWorker) that listens on the server via
   * a socket connection. Recieved data will be processed by 
   * processResponse()
   */
  public void startSocketListenerThread() {
    BufferedReader in = null;
    // create socket connection
    try {
      socket = new Socket("localhost", 5555);
      out = new PrintWriter(socket.getOutputStream(), true);
      in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
      textArea.append("Successfully connected to server\n");
    } catch (UnknownHostException e) {
      textArea.append("Unknown host\n");
    } catch (IOException e) {
      textArea.append("No I/O\n");
    }
    // set up ClientInputWorker thread

    clientInputWorker = new ClientInputWorker(this, in);
    Thread th = new Thread(clientInputWorker);
    th.setDaemon(true);
    th.start();
  }

  //---------------------------------------------------------------
  /**
   * Creates an instance of Client and starts it
   */
  public static void main(String[] args) {
    Client frame = new Client();
    frame.setTitle("Client Program");
    WindowListener l = new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.exit(0);
      }
    };

    frame.addWindowListener(l);
    frame.pack();
    frame.setVisible(true);
    frame.startSocketListenerThread();
  }
}
