/*
 * Created on Aug 7, 2003
*/
package isac.bridge;

/**
 * @author rgradisc
 */

import isac.util.KeyListTactic;
import isac.util.ModSpec;
import isac.util.Tactic;

import java.io.StringReader;

import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Text;
import org.w3c.dom.traversal.DocumentTraversal;
import org.w3c.dom.traversal.NodeFilter;
import org.w3c.dom.traversal.NodeIterator;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

public class XMLParser {

  class ModSpecHandler extends DefaultHandler {
    private ModSpec modSpec;
    private String lastTag;
    private StringBuffer textBuf = new StringBuffer();

    public void startElement(
      String nsURI,
      String localName,
      String qName,
      Attributes attributes)
      throws SAXException {

      lastTag = qName;
    }

    public void endElement(String uri, String localName, String qName) {
      lastTag = qName;
      System.out.println(
        lastTag + "-----" + textBuf.toString().trim() + "-----");
      textBuf.setLength(0);
    }

    public void characters(char[] ch, int start, int length) {
      textBuf.append(ch, start, length);
    }

    public ModSpec getModSpec() {
      return this.modSpec;
    }
  }

  /** Parse eine XML-Datei unter Verwendung von DOM. */

  public static Document DOMparseXMLFile(String path) {
    DOMParser parser = new DOMParser();
    try {
      parser.parse(path);
    } catch (Exception e) {
      e.printStackTrace();
    }
    return parser.getDocument();
  }

  public static Document DOMparseXMLString(String s) {
    InputSource is = new InputSource(new StringReader(s));
    DOMParser parser = new DOMParser();
    try {
      parser.parse(is);
    } catch (Exception e) {
      e.printStackTrace();
    }
    return parser.getDocument();
  }

  private void doIterate(Document document) {
    Node n;
    NodeIterator iter =
      ((DocumentTraversal)document).createNodeIterator(
        document.getFirstChild(),
        NodeFilter.SHOW_ALL,
        null,
        true);

    while ((n = iter.nextNode()) != null) {
      if (n instanceof Element) {
        System.out.println(n.getNodeName() + " " + n.getNodeValue());
      } else if (n instanceof Text) {
        String s = n.getNodeValue().trim();
        if (s.length() != 0) {
          System.out.println("--" + s);
        }
      }
    }
  }

  public ModSpec parseModSpecSAX(String path) throws Exception {
    XMLReader parser =
      XMLReaderFactory.createXMLReader("org.apache.xerces.parsers.SAXParser");
    parser.setContentHandler(new ModSpecHandler());
    parser.parse(path);
    return new ModSpec();
  }

  public ModSpec parseModSpecDOM(String path) {
    Document d = DOMparseXMLFile(path);
    doIterate(d);
    return new ModSpec();
  }

  public static Tactic parseTacticDOM(String tacticXML) {
    KeyListTactic tactic = null;
    Document document = DOMparseXMLString(tacticXML);
    Node n;
    NodeIterator iter =
      ((DocumentTraversal)document).createNodeIterator(
        document.getFirstChild(),
        NodeFilter.SHOW_ALL,
        null,
        true);

    while ((n = iter.nextNode()) != null) {
      if (n instanceof Element) {
        if (n.getNodeName().equals("TACTIC")) {
          tactic =
            new KeyListTactic(
              n.getAttributes().getNamedItem("name").getNodeValue());
        }
      } else if (n instanceof Text) {
        String s = n.getNodeValue().trim();
        if (s.length() != 0 && tactic != null) {
          tactic.addKey(s);
        }
      }
    }
    //tactic.setArgument(sb.toString());
    return tactic;
  }

  public static void main(String[] args) throws Exception {
    XMLParser xp = new XMLParser();
    long before, after;

    for (int i = 0; i < 6; i++) {
      before = System.currentTimeMillis();
      xp.parseModSpecSAX("/radish/hda-local/isac/data/modspec.xml");
      after = System.currentTimeMillis();
      System.out.println(
        "Done parsing: " + (after - before) + " millis needed");
    }
  }
}
