 /****************************************************************** 
* Copyright (c) 2003, Mario Hochreiter, Student of 
* Software Engineering at FH-Hagenberg, Austria. 
* Project member of the isac team at the Institute for 
* Software-Technologie, Graz University of Technology, Austria. 
* 
* Use is subject to license terms.
******************************************************************/

package isac.gui.util;

import isac.util.ICalcIterator;
import isac.wsdialog.DGuide;

import java.rmi.RemoteException;

/**
 * CalcModelHierarchy is the "glue" between the <code>CalculationModel</code>
 * and the <code>CalcIterator</code>. This class is responsible for the
 * representation of a particular <code>CalcElement</code> and it is also
 * responsible to hold information about the position from the 
 * <code>CalcElement</code> in the <code>CalcTree</code>.
 *  
 * @author Mario Hochreiter
 * @version 1.0
 */
public class CalcModelHierarchy {
  /** Always points to the <code>CalcElement</code> that is beeing represented*/
  protected ICalcIterator iterator;

  public CalcModelHierarchy(DGuide dialog) {
    try {
      this.iterator = dialog.iterator();
    } catch (RemoteException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
  }
  
  public CalcModelHierarchy(ICalcIterator iterator) {
    this.iterator = iterator;
  }
    
  public int getChildCount(Object parent) {
    ICalcIterator parentIterator = ((CalcModelHierarchy)parent).iterator;
    ICalcIterator dummyIterator = (ICalcIterator)parentIterator.clone();
    int count = 0;
    dummyIterator.moveLevelDown();
    while (dummyIterator.moveDown()) {
      count++;
    }
    return count;
  }
  
  public Object getChild(Object parent, int index) {
    ICalcIterator parentIterator = ((CalcModelHierarchy)parent).iterator;
    ICalcIterator childIterator = (ICalcIterator)parentIterator.clone();
    CalcModelHierarchy calc = new CalcModelHierarchy(childIterator);
    for (int i = 0; i < index; i++) {
      calc.iterator.moveDown();
    }
    return calc;
  }
  
  /** @return Visual representation of a <code>CalcElement</code> */
  public String toString() {
    //return iterator.getElement().getText();
    return "";
  }
}
