/****************************************************************** 
* Copyright (c) 2003, Mario Hochreiter, Student of 
* Software Engineering at FH-Hagenberg, Austria. 
* Project member of the isac team at the Institute for 
* Software-Technologie, Graz University of Technology, Austria. 
* * 
* Use is subject to license terms.
******************************************************************/
package isac.gui.worksheet;

import isac.gui.treetable.*;
import isac.gui.util.CalcModelHierarchy;
import isac.util.*;
import isac.util.Tactic;
import isac.wsdialog.*;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.rmi.RemoteException;

import javax.swing.*;

/**
 * <code>Worksheet</code> is the component that is responsible for the
 * interaction with the user while doing a calculation. Furthermore it
 * acts as a visual represantiation of mathematical formulas in a calculation
 * tree, that is beeing executed by the isac knowledge interpreter.
 * 
 * The <code>Worksheet</code> is the view of the
 * <code>WorksheetDialogGuide</code>.
 * 
 * The data, that the <code>WorksheetDialogGuide</code> delivers, is 
 * representated in the hierarchical structur <code>JTreeTable</code>.  
 * 
 * @author Mario Hochreiter
 * @version 0.1 15/09/2003
 */
public class Worksheet extends JPanel implements IToUser, ActionListener {
	/** Model for the JTreeTable */
	protected TreeTableModel model;		
	/** Used to represent the model. */
	protected JTreeTable treeTable;
  /** Dialog which is responsible for the communication with the KI*/					
	protected DGuide dialog;
  
  protected JButton nextStep;				
  
  protected JPanel buttonPanel;
  
  //TODO remove: only for testing purposes! 
  JTextArea outputText;					
          
	public Worksheet(DGuide dialogGuide) {
    this.setLayout(new BorderLayout());
    this.buildButtons();
    dialog = dialogGuide;
    outputText = new JTextArea(10, 50);
    this.add(new JScrollPane(outputText), BorderLayout.CENTER);  
	}
  
  private void buildButtons() {
    buttonPanel = new JPanel();
    //TODO localize!!!
    nextStep = new JButton("Next step");
    nextStep.addActionListener(this);
    buttonPanel.add(nextStep);
    this.add(buttonPanel, BorderLayout.SOUTH);
  }
  
  public void writeTestOutput(String output) {
    outputText.append(output);
    outputText.append("\n");
  }
  
  public void setTreeTableModel() {
    CalcModelHierarchy calc = new CalcModelHierarchy(dialog);
    model = new CalculationModel(calc);
    treeTable = new JTreeTable(model);
    this.setLayout(new BorderLayout());
    this.add(new JScrollPane(treeTable), BorderLayout.CENTER);    
  }
	
	//------------- Implementation of IToUser -----------------------
	public void calcChanged(CalcChangedEvent event) {
	
	}
  
  //------------- Implementation of ActionListener ----------------
  public void actionPerformed(ActionEvent e) {
    if (e.getSource() == nextStep) {
      Tactic propTactic;
      try {
        propTactic = dialog.fetchProposedTactic();
        writeTestOutput("Tactic sent to bridge: " + propTactic.getText());
        dialog.setNextTactic(propTactic);    
      } catch (RemoteException e1) {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      } 
    }
  }		
}