(* use"xmlsrc/datatypes.sml";
   use"datatypes.sml";
   *)

val i = indentation;

(** general types: lists,  **)

fun id2xml j ids =
    let fun id2x _ [] = ""
	  | id2x j (s::ss) = (indt j) ^ "<ID> " ^ s ^ " </ID>\n" ^ 
			     (id2x j ss)
    in (indt j) ^ "<KEY>\n" ^ 
       (id2x (j + indentation) ids) ^ 
       (indt j) ^ "</KEY>\n" end;
(* writeln(id2xml 8 ["linear","univariate","equation"]);
        <KEY>
          <ID>linear</ID>
          <ID>univariate</ID>
          <ID>equation</ID>
        </KEY>*)

fun ints2xml j ids =
    let fun int2x _ [] = ""
	  | int2x j (s::ss) = (indt j) ^"<INT> "^ string_of_int s ^" </INT>\n"^
			     (int2x j ss)
    in (indt j) ^ "<INTLIST>\n" ^ 
       (int2x (j + i) ids) ^ 
       (indt j) ^ "</INTLIST>\n" end;
(* writeln(ints2xml 3 [1,2,3]);
   *)


(** isac datatypes **)

fun rule2xml j Erule =
    (indt j ^ "<RULE> empty_rule </RULE>\n"):xml
  | rule2xml j (Thm (thmID, thm)) =
    indt j ^ "<RULE>\n" ^
    indt (j+i) ^ "<THEOREMID>" ^ thmID ^ "</THEOREMID>\n" ^
    term2xml j ((#prop o rep_thm) thm) ^ "\n" ^
    indt j ^ "</RULE>\n"
  | rule2xml j (Calc (op_, _)) =
    indt j ^ "<RULE>\n" ^
    indt (j+i) ^"<CALCULATE> (" ^ op_ ^ ", eval_fun) </CALCULATE>\n" ^
    indt j ^ "</RULE>\n"
  | rule2xml j (Rls_ rls) =
    indt j ^ "<RULE>\n" ^
    indt (j+i) ^"<RULESET> " ^ ((#id o rep_rls) rls) ^ " </RULESET>\n" ^
    indt j ^ "</RULE>\n";
fun rules2xml j [] = ("":xml)
  | rules2xml j (r::rs) = rule2xml j r ^ rules2xml j rs;

fun filterpbl str =
  let fun filt [] = []
        | filt ((s, (t1, t2)) :: ps) = 
	  if str = s then (t1 $ t2) :: filt ps else filt ps
  in filt end;

(*WN.25.8.03: pattern2xml different output with TextIO | writeln !!!
  the version below is for TextIO: terms2xml makes \n !*)
fun pattern2xml j p where_ =
    (case filterpbl "#Given" p of
	[] =>  (indt j) ^ "<GIVEN>  </GIVEN>\n"
      | gis => (indt j) ^ "<GIVEN>\n" ^ terms2xml j gis ^
	       (indt j) ^ "</GIVEN>\n")
    ^ 
    (case where_ of
	 [] =>  (indt j) ^ "<WHERE>  </WHERE>\n"
       | whs => (indt j) ^ "<WHERE>\n" ^ terms2xml j whs ^
		(indt j) ^ "</WHERE>\n")
    ^ 
    (case filterpbl "#Find" p of
	 [] =>  (indt j) ^ "<FIND>  </FIND>\n"
       | fis => (indt j) ^ "<FIND>\n" ^ terms2xml j fis ^
		(indt j) ^ "</FIND>\n")
    ^ 
    (case filterpbl "#Relate" p of
	 [] =>  (indt j) ^ "<RELATE>  </RELATE>\n"
       | res => (indt j) ^ "<RELATE>\n" ^ terms2xml j res ^
		(indt j) ^ "</RELATE>\n");
(*
writeln(pattern2xml 3 ((#ppc o get_pbt)
			 ["squareroot","univariate","equation","test"]) []);
  *)

(*see itm_2item*)
fun itm_2xml j (Cor (dts,_))= 
    (indt j ^"<ITEM>\n"^
    indt (j+i) ^"<CORRECT>\n"^
    term2xml (j+i) (comp_dts' dts)^"\n"^
    indt (j+i) ^"</CORRECT>\n"^
    indt j ^"</ITEM>\n"):xml
  | itm_2xml j (Syn c) =
    indt j ^"<ITEM>\n"^
    indt (j+i) ^"<SYNTAXERROR>\n"^
    indt (j+i) ^c^
    indt (j+i) ^"</SYNTAXERROR>\n"^
    indt j ^"</ITEM>\n"
  | itm_2xml j (Typ c) =
    indt j ^"<ITEM>\n"^
    indt (j+i) ^"<TYPEERROR>\n"^
    indt (j+i) ^c^
    indt (j+i) ^"</TYPEERROR>\n"^
    indt j ^"</ITEM>\n"
  | itm_2xml j (Inc (dts,_)) = 
    indt j ^"<ITEM>\n"^
    indt (j+i) ^"<INCOMPLETE>\n"^
    term2xml (j+i) (comp_dts' dts)^"\n"^
    indt (j+i) ^"</INCOMPLETE>\n"^
    indt j ^"</ITEM>\n"
  | itm_2xml j (Sup dts) = 
    indt j ^"<ITEM>\n"^
    indt (j+i) ^"<SUPERFLUOUS>\n"^
    term2xml (j+i) (comp_dts' dts)^"\n"^
    indt (j+i) ^"</SUPERFLUOUS>\n"^
    indt j ^"</ITEM>\n"
  | itm_2xml j (Mis (d,pid)) =
    indt j ^"<ITEM>\n"^
    indt (j+i) ^"<SUPERFLUOUS>\n"^
    term2xml (j+i) (d $ pid)^"\n"^
    indt (j+i) ^"</SUPERFLUOUS>\n"^
    indt j ^"</ITEM>\n";

(*see terms2xml' fpr \n*)
fun itms2xml _ [] = ""
  | itms2xml j [(_,_,_,_,itm_)] = itm_2xml j itm_
  | itms2xml j (((_,_,_,_,itm_):itm)::itms) = 
    itm_2xml j itm_ ^ itms2xml j itms;

fun precond2xml j (true, term) =
    (indt j ^"<ITEM>\n"^
    indt (j+i) ^"<TRUE>\n"^
    term2xml (j+i) term^"\n"^
    indt (j+i) ^"</TRUE>\n"^
    indt j ^"</ITEM>\n"):xml
  | precond2xml j (false, term) =
    indt j ^"<ITEM>\n"^
    indt (j+i) ^"<FALSE>\n"^
    term2xml (j+i) term^"\n"^
    indt (j+i) ^"</FALSE>\n"^
    indt j ^"</ITEM>\n";

fun preconds2xml _ [] = ("":xml)
  | preconds2xml j (p::ps) = precond2xml j p ^ preconds2xml j ps;

fun model2xml j (itms:itm list) where_ =
    let fun eq4 str (_,_,_,field,_) = str = field
    in  (indt j ^"<MODEL>\n"^
	(case filter (eq4 "#Given") itms of
	     [] =>  (indt (j+i)) ^ "<GIVEN>  </GIVEN>\n"
	   | gis => (indt (j+i)) ^ "<GIVEN>\n" ^ itms2xml (j+2*i) gis ^
		    (indt (j+i)) ^ "</GIVEN>\n")
	^ 
	(case where_ of
	     [] =>  (indt (j+i)) ^ "<WHERE>  </WHERE>\n"
	   | whs => (indt (j+i)) ^ "<WHERE>\n" ^ preconds2xml (j+2*i) whs ^
		    (indt (j+i)) ^ "</WHERE>\n")
	^ 
	(case filter (eq4 "#Find") itms of
	     [] =>  (indt (j+i)) ^ "<FIND>  </FIND>\n"
	   | fis => (indt (j+i)) ^ "<FIND>\n" ^ itms2xml (j+2*i) fis ^
		    (indt (j+i)) ^ "</FIND>\n")
	^ 
	(case filter (eq4 "#Relate") itms of
	     [] =>  (indt (j+i)) ^ "<RELATE>  </RELATE>\n"
	   | res => (indt (j+i)) ^ "<RELATE>\n" ^ itms2xml (j+2*i) res ^
		    (indt (j+i)) ^ "</RELATE>\n")^
	    indt j ^"</MODEL>\n"):xml
    end;
(* writeln(model2xml 3 itms []);
   *)

fun spec2xml j ((dI,pI,mI):spec) =
    (indt j ^"<SPECIFICATION>\n"^ 
    indt (j+i) ^"<THEORY> "^ dI ^" </THEORY>\n"^
    indt (j+i) ^"<PROBLEM>\n"^
    id2xml (j+2*i) pI ^
    indt (j+i) ^"</PROBLEM>\n"^
    indt (j+i) ^"<METHOD>\n"^
    id2xml (j+2*i) mI ^
    indt (j+i) ^"</METHOD>\n"^
    indt j ^"</SPECIFICATION>\n"):xml;

fun modspec2xml j gfr pre spec p_ =
    (indt j ^"<MODSPEC>\n"^ 
     model2xml (j+i) gfr pre ^
     indt (j+i) ^"<BELONGSTO> "^(case p_ of Pbl => "problem" 
					  | Met => "method")^" </BELONGSTO>\n"^ 
     spec2xml (j+i) spec ^
     indt j ^"</MODSPEC>\n"):xml;

fun sub2xml j (id, value) = 
    (indt j ^"<PAIR>\n"^ 
     indt j ^"  <VARIABLE>\n"^ 
     term2xml (j+i) id ^ "\n" ^
     indt j ^"  </VARIABLE>\n" ^
     indt j ^"  <VALUE>\n"^ 
     term2xml (j+i) value ^ "\n" ^
     indt j ^"  </VALUE>\n" ^
     indt j ^"</PAIR>\n"):xml;

fun subs2xml j (subs:subs) = 
    (indt j ^"<SUBSTITUTION>\n"^
     foldl op^ ("", map (sub2xml (j+i)) 
			(subs2subst (assoc_thy "Isac.thy") subs)) ^
     indt j ^"</SUBSTITUTION>\n"):xml;
(* val subs = [(str2term "bdv", str2term "x")];
   writeln(subs2xml 0 subs);
   *)

fun pos_2xml j pos_ =
    (indt j) ^ "<POS> " ^  pos_2str pos_ ^ " </POS>\n";

fun pos'2xml j (pos, pos_) =
    indt j ^ "<POSITION>\n" ^ 
    ints2xml (j + i) pos ^ 
    pos_2xml (j + i) pos_ ^ 
    indt j ^ "</POSITION>\n";
(* writeln(pos'2xml 3 ([1,2,3], Pbl));
   *)



fun thmstr2xml j str = ((((term2xml j) o str2term) str)^"\n"):xml;

fun thm'2xml j ((ID, form):thm') =
    (indt j ^"<THEOREM>\n"^
    indt (j+i) ^"<ID> "^ID^" </ID>\n"^
    indt (j+i) ^"<FORMULA>\n"^
    thmstr2xml (j+i) form^
    indt (j+i) ^"</FORMULA>\n"^
    indt j ^"</THEOREM>\n"):xml;

fun tac2xml j (Subproblem (dI, pI)) =
    (indt j ^"<TACTIC name=\"Subproblem\">\n"^    
    indt (j+i) ^"<THEORY> "^ dI ^" </THEORY>\n"^
    indt (j+i) ^"<PROBLEM>\n"^
    id2xml (j+2*i) pI^
    indt (j+i) ^"</PROBLEM>\n"^    
    indt j ^"</TACTIC>\n"):xml
  | tac2xml j (Model_Problem pI) =
    (indt j ^"<TACTIC name=\"Model_Problem\">\n"^    
    id2xml (j+i) pI^    
    indt j ^"</TACTIC>\n"):xml
  | tac2xml j (Refine_Tacitly pI) =
    (indt j ^"<TACTIC name=\"Refine_Tacitly\">\n"^    
    id2xml (j+i) pI^    
    indt j ^"</TACTIC>\n"):xml

  | tac2xml j (Add_Given ct) =
    (indt j ^"<TACTIC name=\"Add_Given\">\n"^    
    cterm2xml (j+i) ct^    
    indt j ^"</TACTIC>\n"):xml
  | tac2xml j (Add_Find ct) =
    (indt j ^"<TACTIC name=\"Add_Find\">\n"^    
    cterm2xml (j+i) ct^    
    indt j ^"</TACTIC>\n"):xml
  | tac2xml j (Add_Relation ct) =
    (indt j ^"<TACTIC name=\"Add_Relation\">\n"^    
    cterm2xml (j+i) ct^    
    indt j ^"</TACTIC>\n"):xml

  | tac2xml j (Specify_Theory ct) =
    (indt j ^"<TACTIC name=\"Specify_Theory\">\n"^    
    cterm2xml (j+i) ct^    
    indt j ^"</TACTIC>\n"):xml
  | tac2xml j (Specify_Problem ct) =
    (indt j ^"<TACTIC name=\"Specify_Problem\">\n"^    
    id2xml (j+i) ct^    
    indt j ^"</TACTIC>\n"):xml
  | tac2xml j (Specify_Method ct) =
    (indt j ^"<TACTIC name=\"Specify_Method\">\n"^    
    id2xml (j+i) ct^    
    indt j ^"</TACTIC>\n"):xml
  | tac2xml j (Apply_Method mI) =
    (indt j ^"<TACTIC name=\"Apply_Method\">\n"^    
    id2xml (j+i) mI^    
    indt j ^"</TACTIC>\n"):xml

  | tac2xml j (Rewrite thm') =
    (indt j ^"<TACTIC name=\"Rewrite\">\n"^    
    thm'2xml (j+i) thm'^    
    indt j ^"</TACTIC>\n"):xml
  | tac2xml j (Rewrite_Inst (subs, thm')) =
    (indt j ^"<TACTIC name=\"Rewrite_Inst\">\n"^    
    subs2xml (j+i) subs^
    thm'2xml (j+i) thm'^    
    indt j ^"</TACTIC>\n"):xml
  | tac2xml j (Rewrite_Set rls') =
    (indt j ^"<TACTIC name=\"Rewrite_Set\">\n"^    
    indt (j+i) ^"<RULESET> "^ rls' ^" </RULESET>\n"^    
    indt j ^"</TACTIC>\n"):xml
  | tac2xml j (Rewrite_Set_Inst (subs, rls')) =
    (indt j ^"<TACTIC name=\"Rewrite_Set_Inst\">\n"^    
    indt (j+i) ^"<RULESET> "^ rls' ^" </RULESET>\n"^
    subs2xml (j+i) subs^    
    indt j ^"</TACTIC>\n"):xml

  | tac2xml j (Check_elementwise ct) =
    (indt j ^"<TACTIC name=\"Check_elementwise\">\n"^    
    term2xml (j+i) 
	     (str2term ct
	      (*FIXME.WN.9.03: redesign cterm' for xml-output*)) ^ "\n"^    
    indt j ^"</TACTIC>\n"):xml
  | tac2xml j (Check_Postcond pI) =
    (indt j ^"<TACTIC name=\"Check_Postcond\">\n"^    
    id2xml (j+i) pI^    
    indt j ^"</TACTIC>\n"):xml

  | tac2xml j tac = raise error ("tac2xml: not impl. for "^tac2str tac);

